/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package assistant.app;

import assistant.Assistant;
import assistant.Assistant.SECTION;
import i18n.I18N;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import storybook.ui.Ui;

/**
 *
 * @author favdb
 */
public class AssistantMenu extends JMenuBar {

	private static final String TT = "AssistantMenu";

	private final AssistantApp app;
	private JMenuItem fileMenuSave;
	private JMenuItem fileMenuSaveAs;

	public AssistantMenu(AssistantApp app) {
		super();
		this.app = app;
		initialize();
	}

	private void initialize() {
		AssitantAction appAction = app.getAppAction();
		JMenu fileMenu = Ui.addMenu(I18N.getMsg("file"));
		//fileMenu.add(Ui.addMenuItem(I18N.getMsg("file.new"), e -> appAction.fileNew()));
		fileMenu.add(Ui.addMenuItem(I18N.getMsg("file.open"), e -> appAction.fileOpen()));
		fileMenuSave = Ui.addMenuItem(I18N.getMsg("file.save"), e -> appAction.fileSave());
		fileMenu.add(fileMenuSave);
		fileMenuSaveAs = Ui.addMenuItem(I18N.getMsg("file.save.as"), e -> appAction.fileSaveAs());
		fileMenu.add(fileMenuSaveAs);
		fileMenu.add(Ui.addMenuItem(I18N.getMsg("exit"), e -> appAction.fileExit()));
		add(fileMenu);

		/* no edit allowed
		JMenu editMenu = Ui.addMenu(I18N.getMsg("edit"));
		editMenu.add(Ui.addMenuItem(I18N.getMsg("edit.preferences"), e -> appAction.editPreferences()));
		add(editMenu);
		 */
		String tmenu = "test";
		String exLocation = "<location><Saison>Automne</Saison></location>";
		String exPerson = "<person><Physique><Bouche>Noir</Bouche><Cheveux>Auburn</Cheveux></Physique></person>";
		String exItem = "<item><Forme>Carré</Forme><Matière>bois</Matière></item>";
		String exScene = "<book><scenetext><Situation>Vengeance</Situation></scenetext></book>";
		String exPlot = "";

		JMenu testMenu = Ui.addMenu(I18N.getMsg(tmenu));
		if (Assistant.getXml().getSection(SECTION.BOOK) != null) {
			testMenu.add(Ui.addMenuItem(
					I18N.getMsg(tmenu + "." + SECTION.BOOK.toString()),
					e -> appAction.testDialog(SECTION.BOOK, exScene)));
		}
		if (Assistant.getXml().getSection(SECTION.PERSON).getLength() > 0) {
			testMenu.add(Ui.addMenuItem(
					I18N.getMsg(tmenu + "." + SECTION.PERSON.toString()),
					e -> appAction.testDialog(SECTION.PERSON, exPerson)));
		}
		if (Assistant.getXml().getSection(SECTION.LOCATION).getLength() > 0) {
			testMenu.add(Ui.addMenuItem(
					I18N.getMsg(tmenu + "." + SECTION.LOCATION.toString()),
					e -> appAction.testDialog(SECTION.LOCATION, exLocation)));
		}
		if (Assistant.getXml().getSection(SECTION.ITEM).getLength() > 0) {
			testMenu.add(Ui.addMenuItem(
					I18N.getMsg(tmenu + "." + SECTION.ITEM.toString()),
					e -> appAction.testDialog(SECTION.ITEM, exItem)));
		}
		if (Assistant.getXml().getSection(SECTION.PLOT).getLength() > 0) {
			testMenu.add(Ui.addMenuItem(
					I18N.getMsg(tmenu + "." + SECTION.PLOT.toString()),
					e -> appAction.testDialog(SECTION.PLOT, exPlot)));
		}
		add(testMenu);

		JMenu helpMenu = Ui.addMenu(I18N.getMsg("help"));
		//helpMenu.add(Ui.addMenuItem(I18N.getMsg("help.contents"), e -> appAction.helpContents()));
		helpMenu.add(Ui.addMenuItem(I18N.getMsg("help.about"), e -> appAction.helpAbout()));
		add(helpMenu);
	}

	public void setSaveEnable(boolean b) {
		//LOG.trace(TT + ".setSaveEnable(b=" + (b ? "true" : "false") + ")");
		fileMenuSave.setEnabled(false);
		if (Assistant.getXml().getSave() != null) {
			fileMenuSave.setEnabled(app.isModified());
		}
		fileMenuSaveAs.setEnabled(app.isModified());
	}

}
