/*
 * Copyright (C) 2019 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.edit;

import storybook.ui.Ui;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import org.miginfocom.swing.MigLayout;
import storybook.Const;
import i18n.I18N;
import storybook.model.book.Book;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Relationship;
import storybook.model.hbn.entity.Scene;

/**
 *
 * @author favdb
 */
public class EditRelation extends AbstractEditor {

	private JComboBox cbSceneStart;
	private JComboBox cbSceneEnd;
	private CheckBoxPanel lsPersons;
	private CheckBoxPanel lsLocations;
	private CheckBoxPanel lsItems;

	public EditRelation(Editor m, AbstractEntity e) {
		super(m, e, "110");
		initAll();
	}

	@Override
	public void initUpper() {
		Relationship relation = (Relationship) entity;
		Scene d = relation.getStartScene();
		cbSceneStart = Ui.initAutoCombo(panel, mainFrame, "scene.start", Book.TYPE.SCENE, d, null, !MANDATORY, !NEW, EMPTY);
		Scene f = relation.getEndScene();
		cbSceneEnd = Ui.initAutoCombo(panel, mainFrame, "scene.end", Book.TYPE.SCENE, f, null, !MANDATORY, !NEW, EMPTY);
		JPanel links = new JPanel(new MigLayout());
		lsPersons = Ui.initCbList(links, mainFrame, Book.TYPE.PERSON, "persons", relation.getPersons(), !MANDATORY, null);
		if (relation.hasPerson1()) {
			lsPersons.selectEntity(relation.getPerson1());
		}
		if (relation.hasPerson2()) {
			lsPersons.selectEntity(relation.getPerson2());
		}
		lsLocations = Ui.initCbList(links, mainFrame, Book.TYPE.LOCATION, "locations", relation.getLocations(), !MANDATORY, null);
		lsItems = Ui.initCbList(links, mainFrame, Book.TYPE.ITEM, "items", relation.getItems(), !MANDATORY, null);
		tab1.add(I18N.getMsg("links"), links);
	}

	@Override
	public boolean verifier() {
		resetError();
		if (cbSceneStart.getSelectedItem() != null
				&& cbSceneEnd.getSelectedItem() == null) {
			errorMsg(cbSceneEnd, Const.ERROR_MISSING);
		}
		if (lsPersons.getSelectedEntities().isEmpty()
				&& lsLocations.getSelectedEntities().isEmpty()
				&& lsItems.getSelectedEntities().isEmpty()) {
			errorMsg(lsPersons, "error.must_select");
		}
		return (msgError.isEmpty());
	}

	@Override
	public void apply() {
		Relationship relation = (Relationship) entity;
		if (cbSceneStart.getSelectedIndex() > 0) {
			relation.setStartScene((Scene) cbSceneStart.getSelectedItem());
		} else {
			relation.setStartScene(null);
		}
		if (cbSceneEnd.getSelectedIndex() > 0) {
			relation.setEndScene((Scene) cbSceneEnd.getSelectedItem());
		} else {
			relation.setEndScene(null);
		}
		relation.setPerson1();
		relation.setPerson2();
		List<AbstractEntity> ls;
		{
			ls = lsPersons.getSelectedEntities();
			List<Person> lsc = new ArrayList<>();
			for (AbstractEntity e : ls) {
				lsc.add((Person) e);
			}
			relation.setPersons(lsc);
		}
		{
			ls = lsLocations.getSelectedEntities();
			List<Location> lsc = new ArrayList<>();
			for (AbstractEntity e : ls) {
				lsc.add((Location) e);
			}
			relation.setLocations(lsc);
		}
		{
			ls = lsItems.getSelectedEntities();
			List<Item> lsc = new ArrayList<>();
			for (AbstractEntity e : ls) {
				lsc.add((Item) e);
			}
			relation.setItems(lsc);
		}
		super.apply();
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

}
