/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.entity;

import storybook.model.book.Book;

/**
 *
 * @author favdb
 */
public class Episode extends AbstractEntity {

	private Integer number;
	private Strand strand = null;
	private Scene scene = null;
	private Chapter chapter = null;

	public Episode() {
		this(-1);
	}

	public Episode(int number) {
		super(Book.TYPE.EPISODE, "010");
		setNumber(number);
	}

	public Integer getNumber() {
		return number;
	}

	public void setNumber(Integer number) {
		this.number = (number == null ? -1 : number);
	}

	public Strand getStrand() {
		return strand;
	}

	public void setStrand(Strand strand) {
		this.strand = strand;
	}

	public Chapter getChapter() {
		return chapter;
	}

	public void setChapter(Chapter entity) {
		this.chapter = entity;
	}

	public boolean hasChapter() {
		return chapter != null;
	}

	public Scene getScene() {
		return scene;
	}

	public void setScene(Scene entity) {
		this.scene = entity;
	}

	public boolean hasScene() {
		return scene != null;
	}

	public void setLink(AbstractEntity entity) {
		if (entity instanceof Chapter) {
			chapter = (Chapter) entity;
			scene = null;
		} else if (entity instanceof Scene) {
			chapter = null;
			scene = (Scene) entity;
		} else {
			chapter = null;
			scene = null;
		}
	}

	public AbstractEntity getLink() {
		if (hasChapter()) {
			return chapter;
		}
		if (hasScene()) {
			return scene;
		}
		return null;
	}

	public boolean hasLink() {
		return hasChapter() || hasScene();
	}

	public void setStrandNotes(Strand strand, String val) {
		setStrand(strand);
		setNotes(val);
	}

}
