/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.toolkit;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import storybook.model.hbn.entity.AbstractEntity;

/**
 *
 * @author favdb
 */
public class LOG {

	private static boolean bTrace = false, bHibernate = false;

	public static void setTrace() {
		bTrace = true;
	}

	public static void setTrace(boolean btrace) {
		bTrace = btrace;
	}

	public static void setTraceHibernate() {
		bHibernate = true;
	}

	private LOG() {
		//empty
	}

	public static void init(boolean trace, boolean hibernate) {
		bTrace = trace;
		bHibernate = hibernate;
	}

	public static void log(String msg) {
		DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss.SSS");
		System.out.println(dateFormat.format(new Date()) + " " + msg);
	}

	public static boolean getTrace() {
		return bTrace;
	}

	public static boolean getTraceHibernate() {
		return bHibernate;
	}

	public static void trace(String msg) {
		if (bTrace) {
			log(msg);
		}
	}

	public static String traceEntity(AbstractEntity entity) {
		return (entity == null ? "null" : entity.getName());
	}

	public static void err(String txt, Exception... e) {
		if (e != null && e.length > 0) {
			System.err.println(txt + " Exception:" + e[0].getMessage());
			e[0].printStackTrace(System.err);
		} else {
			System.err.println(txt);
		}
	}

	public static void setLogging() {
		if (getTraceHibernate()) {
			enableLogging();
		} else {
			disableLogging();
		}
	}

	public static void disableLogging() {
		LOG.trace("Disable all logging");
		LogManager logManager = LogManager.getLogManager();
		Logger logger = logManager.getLogger("");
		logger.setLevel(Level.OFF);
	}

	public static void enableLogging() {
		LOG.trace("Enable all logging to INFO");
		LogManager logManager = LogManager.getLogManager();
		Logger logger = logManager.getLogger("");
		logger.setLevel(Level.INFO);
	}

}
