/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package storybook.ui;

import i18n.I18N;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.util.ArrayList;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconButton;
import resources.icons.IconUtil;
import shef.ShefEditor;
import storybook.App;
import storybook.edit.AbstractEditor;
import storybook.edit.CheckBoxPanel;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.book.BookUtil;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.DB_DATA.DATA;
import storybook.model.hbn.entity.Relationship;
import storybook.toolkit.ListUtil;
import storybook.toolkit.swing.FontUtil;
import storybook.toolkit.swing.SwingUtil;
import storybook.toolkit.swing.js.JSLabel;
import storybook.ui.combobox.ComboUtil;
import storybook.ui.renderer.lcr.LCRStatus;

/**
 *
 * @author favdb
 */
public class Ui {

	private static final String TT = "Ui";

	private Ui() {
		throw new IllegalStateException("Utility class");
	}

	public static final Dimension MINIMUM_SIZE = new Dimension(440, 120),
			MAXIMUM_SIZE = new Dimension(800, 600),
			PREF_SIZE = new Dimension(750, 560);
	public static final boolean MANDATORY = true,
			NEW = true,
			ALL = true,
			EMPTY = true,
			HIDE = true,
			INFO = true,
			NEWTAB = true,
			GROW = true,
			TIME = true,
			TOP = true,
			WRAP = true;

	public static void addField(JPanel panel, String title, String top,
			JComponent comp, boolean grow, boolean mandatory, JTabbedPane tab) {
		if (tab != null) {
			tab.add(comp, I18N.getMsg(title));
		} else {
			JLabel lb = new JLabel(I18N.getColonMsg(title));
			if (mandatory) {
				lb.setFont(FontUtil.getBoldFont(lb.getFont()));
			}
			String c = top;
			if (!top.contains(MIG.WRAP)) {
				c = top + ", " + MIG.RIGHT;
			}
			String strgrow = "";
			if (top.contains(MIG.WRAP)) {
				strgrow = MIG.get(MIG.WRAP, MIG.SPAN);
			}
			if (grow) {
				strgrow = MIG.GROW;
			}
			if (!title.isEmpty()) {
				panel.add(lb, c);
			}
			panel.add(comp, strgrow);
		}
	}

	public static void addLabel(JPanel panel, String title, boolean isTop, boolean mandatory) {
		String top = "";
		if (isTop) {
			top = "top,";
		}
		JSLabel lb = new JSLabel(I18N.getColonMsg(title));
		lb.setFont(App.getInstance().fontGetDefault());
		if (mandatory) {
			lb.setFont(FontUtil.getBoldFont(lb.getFont()));
		}
		panel.add(lb, top + MIG.RIGHT);
	}

	public static JTextField initStringField(JPanel p, String title, int len,
			String val,
			boolean isMandatory, boolean isInfo) {
		JTextField tf = new JTextField();
		tf.setName(title);
		tf.setText(val);
		boolean grow = false;
		if (len > 0) {
			tf.setColumns(len);
		} else {
			grow = true;
		}
		if (isInfo) {
			tf.setEditable(false);
		}
		addField(p, title, "", tf, grow, isMandatory, null);
		return (tf);
	}

	public static JTextField getStringField(DATA data, int len, String val, boolean isInfo) {
		return getStringField(data.toString(), len, val, isInfo);
	}

	public static JTextField getStringField(String name, int len, String val, boolean isInfo) {
		JTextField tf = new JTextField();
		tf.setName(name);
		tf.setText(val);
		if (len > 0) {
			tf.setColumns(len);
		}
		if (isInfo) {
			tf.setEditable(false);
		}
		return (tf);
	}

	public static JTextField initStringField(JPanel p, DATA data, int len, String val, boolean mandatory, boolean info) {
		return initStringField(p, data.i18n(), len, val, mandatory, info);
	}

	public static JTextField initIntegerField(JPanel p, DATA data, int len, Integer val, boolean mandatory, boolean info) {
		return initIntegerField(p, data.i18n(), len, val, mandatory, info);
	}

	public static JTextField initIntegerField(JPanel p, String name, int len, Integer val, boolean mandatory, boolean info) {
		if (val == null) {
			val = 0;
		}
		String s = val.toString();
		if (val == 0 && mandatory == false) {
			s = "";
		}
		return (initStringField(p, name, len, s, mandatory, info));
	}

	public static ShefEditor initHtmlEditor(JPanel panel, MainFrame mainFrame,
			String name, String val, int len,
			boolean mandatory, JTabbedPane tab, boolean full) {
		ShefEditor ta = new ShefEditor("none", val);
		ta.setName(name);
		ta.setMinimumSize(new Dimension(200, 80));
		ta.setPreferredSize(new Dimension(800, 600));
		ta.setMaximumSize(new Dimension(1600, 1200));
		if (tab == null) {
			//ta.setPreferredSize(new Dimension(400, 280));
		}
		addField(panel, name, MIG.TOP, ta, GROW, mandatory, tab);
		return ta;
	}

	@SuppressWarnings("unchecked")
	public static JComboBox initComboBox(JPanel panel, AbstractEditor caller,
			String name,
			List<String> list,
			String toSel,
			boolean mandatory, boolean addNew, boolean addEmpty) {
		JComboBox combo = new JComboBox();
		//combo.setPreferredSize(new Dimension(250, 26));
		combo.setName(name);
		if (addEmpty) {
			combo.addItem("");
		}
		for (String str : list) {
			combo.addItem(str);
		}
		if (toSel != null && !toSel.isEmpty()) {
			combo.setSelectedItem(toSel);
		}
		if (addNew) {
			JPanel p = new JPanel(new MigLayout(MIG.INS0));
			JButton bt = new IconButton("btNew", ICONS.K.NEW, caller);
			p.add(combo);
			p.add(bt, MIG.SHRINK);
			addLabel(panel, name, !TOP, mandatory);
			panel.add(p, MIG.LEFT);
		} else {
			addField(panel, name, "", combo, !GROW, mandatory, null);
		}
		return (combo);
	}

	public static JComboBox initComboBox(JPanel panel, AbstractEditor caller,
			String name, Book.TYPE objtype,
			AbstractEntity toSel, AbstractEntity toHide,
			boolean mandatory, boolean addNew, boolean addEmpty) {
		JComboBox cb = new JComboBox();
		cb.setFont(App.getInstance().fontGetDefault());
		cb.setName(name);
		ComboUtil.fillEntity(caller.mainFrame, cb, objtype, toSel, toHide,
				(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "0");
		if (addNew) {
			JPanel p = new JPanel(new MigLayout(MIG.INS0));
			JButton bt = new JButton(IconUtil.getIconSmall(ICONS.K.NEW));
			SwingUtil.setForcedSize(bt, IconUtil.getDefDim());
			bt.addActionListener((ActionEvent e) -> {
				AbstractEntity newEntity = BookUtil.getNewEntity(objtype);
				boolean rc = caller.mainFrame.showEditorAsDialog(newEntity, bt);
				if (rc != true) {
					ComboUtil.fillEntity(caller.mainFrame, cb, objtype, newEntity, toHide,
							(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "1");
				}
			});
			p.add(cb);
			p.add(bt, MIG.SHRINK);
			addLabel(panel, name, !TOP, mandatory);
			panel.add(p, MIG.LEFT);
		} else {
			addField(panel, name, "", cb, !GROW, mandatory, null);
		}
		return (cb);
	}

	public static JComboBox initComboBox(JPanel panel, AbstractEditor caller,
			String name, Book.TYPE objtype,
			Long toSel, AbstractEntity toHide,
			boolean mandatory, boolean addNew, boolean addEmpty) {
		JComboBox combo = new JComboBox();
		combo.setFont(App.getInstance().fontGetDefault());
		//combo.setPreferredSize(new Dimension(250, 26));
		combo.setName(name);
		ComboUtil.fillEntity(caller.mainFrame, combo, objtype, toSel, toHide,
				(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "0");
		if (addNew) {
			JPanel p = new JPanel(new MigLayout(MIG.INS0));
			JButton bt = new JButton(IconUtil.getIconSmall(ICONS.K.NEW));
			SwingUtil.setForcedSize(bt, new Dimension(22, 22));
			bt.addActionListener((ActionEvent e) -> {
				AbstractEntity newEntity = BookUtil.getNewEntity(objtype);
				boolean rc = caller.mainFrame.showEditorAsDialog(newEntity, bt);
				if (rc != true) {
					ComboUtil.fillEntity(caller.mainFrame, combo, objtype, newEntity, toHide,
							(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "1");
				}
			});
			p.add(combo);
			p.add(bt, MIG.SHRINK);
			addLabel(panel, objtype.toString(), !TOP, mandatory);
			panel.add(p, MIG.LEFT);
		} else {
			addField(panel, name, "", combo, !GROW, mandatory, null);
		}
		return (combo);
	}

	public static JComboBox getComboBox(JPanel panel, AbstractEditor caller,
			Book.TYPE objtype,
			AbstractEntity toSel, AbstractEntity toHide,
			boolean mandatory, boolean addNew, boolean addEmpty) {
		return getComboBox(panel, caller, objtype.toString(), objtype, toSel, toHide, mandatory, addNew, addEmpty);

	}

	public static JComboBox getComboBox(JPanel panel, AbstractEditor caller,
			String name, Book.TYPE objtype,
			AbstractEntity toSel, AbstractEntity toHide,
			boolean mandatory, boolean addNew, boolean addEmpty) {
		JComboBox combo = new JComboBox();
		combo.setFont(App.getInstance().fontGetDefault());
		combo.setName(name);
		ComboUtil.fillEntity(caller.mainFrame, combo, objtype, toSel, toHide,
				(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "0");
		if (addNew) {
			JPanel p = new JPanel(new MigLayout(MIG.INS0));
			JButton bt = new JButton(IconUtil.getIconSmall(ICONS.K.NEW));
			SwingUtil.setForcedSize(bt, IconUtil.getDefDim());
			bt.addActionListener((ActionEvent e) -> {
				AbstractEntity newEntity = BookUtil.getNewEntity(objtype);
				boolean rc = caller.mainFrame.showEditorAsDialog(newEntity, bt);
				if (rc != true) {
					ComboUtil.fillEntity(caller.mainFrame, combo, objtype, newEntity, toHide,
							(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "1");
				}
			});
			p.add(combo);
			p.add(bt, MIG.SHRINK);
			addLabel(panel, objtype.toString(), !TOP, mandatory);
			panel.add(p, MIG.LEFT);
		} else {
			panel.add(combo, MIG.GROWX);
		}
		return (combo);
	}

	/**
	 * initialize a JCheckBox
	 *
	 * @param panel: JPanel where add the JCheckBox, if null the JCheckBox isn't added anywhere just
	 * return the JCheckBox
	 * @param name: internal name of the JCheckBox
	 * @param text: text for the JCheckBox
	 * @param check: true if the initial state would be selected
	 * @param isMandatory: true for changing the font to the bold one, tha indicate te JCheckBox is
	 * mandatory
	 * @param top: optional, MigLayout directive for positioning the JCheckOX
	 * @return an initialized JCheckBox
	 */
	public static JCheckBox initCheckBox(JPanel panel,
			String name,
			String text,
			boolean check,
			boolean isMandatory,
			String... top) {
		JCheckBox cb = new JCheckBox();
		cb.setName(name);
		cb.setText(I18N.getMsg(text));
		cb.setSelected(check);
		if (isMandatory) {
			cb.setFont(FontUtil.getBoldFont());
		}
		if (panel != null) {
			if (top.length > 0) {
				panel.add(cb, top[0]);
			} else {
				panel.add(cb);
			}
		}
		return (cb);
	}

	public static JCheckBox initCheckBox(String name, String title, boolean sel, ActionListener action) {
		//App.trace(TT+".initCheckbox("+title+", sel="+(sel?"true":"false")+")");
		JCheckBox cb = new JCheckBox();
		cb.setName(name);
		cb.setText(I18N.getMsg(title));
		cb.setOpaque(false);
		cb.setSelected(sel);
		cb.addActionListener(action);
		return (cb);
	}

	@SuppressWarnings("unchecked")
	public static JComboBox initComboBox(String name, String tooltip, List list, AbstractEntity toSel,
			boolean isEmpty, boolean isAll, ActionListener... action) {
		//App.trace(TT+".comboboxInit(title="+title+"tooltip, list, toSel, isEmpty, isAll)");
		JComboBox cb = new JComboBox();
		cb.setName(name);
		if (tooltip != null && !tooltip.isEmpty()) {
			cb.setToolTipText(I18N.getMsg(tooltip));
		}
		cb.setOpaque(false);
		if (isEmpty) {
			cb.addItem("");
		}
		if (isAll) {
			cb.addItem(I18N.getMsg("all"));
		}
		if (list != null && !list.isEmpty()) {
			list.forEach(e -> cb.addItem(e));
			if (toSel != null) {
				cb.setSelectedItem(toSel);
			} else {
				cb.setSelectedIndex(0);
			}
		}
		if (action != null && action.length > 0) {
			cb.addActionListener(action[0]);
		}
		return (cb);
	}

	@SuppressWarnings("unchecked")
	public static void fillComboBox(JComboBox cb, List list, boolean isEmpty, boolean isAll, ActionListener... action) {
		//App.trace(TT+".fillComboBox(cb, list, toSel, isEmpty, isAll)");
		int toSel = cb.getSelectedIndex();
		if (action != null && action.length > 0) {
			cb.removeActionListener(action[0]);
		}
		cb.removeAllItems();
		if (isEmpty) {
			cb.addItem("");
		}
		if (isAll) {
			cb.addItem(I18N.getMsg("all"));
		}
		if (list != null && !list.isEmpty()) {
			list.forEach((e) -> {
				AbstractEntity entity = (AbstractEntity) e;
				cb.addItem(entity);
			});
		}
		cb.setSelectedIndex(toSel);
		if (action != null && action.length > 0) {
			cb.addActionListener(action[0]);
		}
	}

	/**
	 * initialize a JRadioButton
	 *
	 * @param panel: JPanel where add the JCheckBox, if null the JCheckBox isn't added anywhere
	 * @param name: internal name of the JCheckBox
	 * @param text: text for the JCheckBox
	 * @param check: true if the initial state would be selected
	 * @param isMandatory: true for changing the font to the bold one, tha indicate te JCheckBox is
	 * mandatory
	 * @param top: optional, MigLayout directive for positioning the JCheckOX
	 * @return an initialized JCheckBox
	 */
	public static JRadioButton initRadioButton(JPanel panel,
			String text,
			boolean check,
			boolean isMandatory,
			String... top) {
		JRadioButton cb = new JRadioButton();
		cb.setName(text);
		cb.setText(I18N.getMsg(text));
		cb.setSelected(check);
		if (isMandatory) {
			cb.setFont(FontUtil.getBoldFont());
		}
		if (panel != null) {
			if (top.length > 0) {
				panel.add(cb, top[0]);
			} else {
				panel.add(cb);
			}
		}
		return (cb);
	}

	public static JComboBox initAutoCB(JPanel panel, MainFrame mainFrame, String title,
			Book.TYPE objtype, AbstractEntity toSel, boolean addNew) {
		return (Ui.initAutoCombo(panel, mainFrame, title, objtype, toSel, null, !MANDATORY, addNew, EMPTY));
	}

	public static JComboBox initAutoComBo(JPanel panel, MainFrame mainFrame, DATA title,
			Book.TYPE objtype, AbstractEntity toSel, boolean mandatory, boolean addEmpty, boolean addNew, boolean... growx) {
		return (Ui.initAutoCombo(panel, mainFrame, title.toString(), objtype, toSel, null, mandatory, addNew, addEmpty));
	}

	/**
	 * combobox for a String list, with mandatory, empty allowed and optional grow
	 *
	 * @param panel
	 * @param title
	 * @param list
	 * @param toSel
	 * @param mandatory
	 * @param addEmpty
	 * @param growx
	 * @return
	 */
	public static JComboBox initAutoCombo(JPanel panel,
			DATA title,
			List<String> list, String toSel,
			boolean mandatory, boolean addEmpty, boolean... growx) {
		return initAutoCombo(panel, title.i18n(), list, toSel, mandatory, addEmpty, growx);
	}

	@SuppressWarnings("unchecked")
	public static JComboBox initAutoCombo(JPanel panel,
			String title,
			List<String> list, String toSel,
			boolean mandatory, boolean addEmpty, boolean... growx) {
		//App.trace(TT+".initAutoCombo(panel"
		//		+ ", title="+title
		//		+ ", list size="+list.size()
		//		+ ", toSel=\""+(toSel==null?"null":toSel)+"\""
		//		+ ", mandatory="+(mandatory?"true":"false")
		//		+ ", addEmpty="+(addEmpty?"true":"false")
		//				+ ")");
		JComboBox combo = new JComboBox();
		combo.setName(title);
		combo.setMinimumSize(new Dimension(150, 20));
		combo.setEditable(true);
		if (addEmpty) {
			combo.addItem("");
		}
		list.forEach((str) -> {
			if (!str.isEmpty()) {
				combo.addItem(str);
			}
		});
		if (toSel != null && !toSel.isEmpty()) {
			combo.setSelectedItem(toSel);
		}
		if (growx == null) {
			addField(panel, title, "", combo, !GROW, mandatory, null);
		} else {
			addField(panel, title, "", combo, GROW, mandatory, null);
		}
		return (combo);
	}

	public static JComboBox initAutoCombo(JPanel panel, MainFrame mainFrame,
			DATA title,
			Book.TYPE objtype,
			AbstractEntity toSel, AbstractEntity toHide,
			boolean mandatory, boolean addNew, boolean addEmpty) {
		return initAutoCombo(panel, mainFrame, title.i18n(), objtype, toSel, toHide, mandatory, addNew, addEmpty);
	}

	public static JComboBox initAutoCombo(JPanel panel, MainFrame mainFrame,
			String title,
			Book.TYPE objtype,
			AbstractEntity toSel, AbstractEntity toHide,
			boolean mandatory, boolean addNew, boolean addEmpty) {
		JComboBox combo = new JComboBox();
		combo.setName(title);
		combo.setMinimumSize(new Dimension(150, 20));
		ComboUtil.fillEntity(mainFrame, combo, objtype, toSel, toHide,
				(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "0");
		JButton bt = SwingUtil.getIconButton("new", null);
		bt.addActionListener((ActionEvent e) -> {
			AbstractEntity newEntity = BookUtil.getNewEntity(objtype);
			boolean rc = mainFrame.showEditorAsDialog(newEntity, bt);
			if (rc != true) {
				ComboUtil.fillEntity(mainFrame, combo, objtype, newEntity, toHide,
						(addNew ? "1" : "0") + (addEmpty ? "1" : "0") + "0");
			}
		});
		bt.setVisible(addNew);
		JPanel p = new JPanel(new MigLayout("ins 2 n 0 n"));
		p.add(combo);
		p.add(bt);
		addField(panel, title, "", p, !Ui.GROW, mandatory, null);
		return (combo);
	}

	@SuppressWarnings("unchecked")
	public static JPanel initListBox(JPanel panel, MainFrame mainFrame, Book.TYPE type, String title,
			List<?> list, AbstractEntity tosel, ActionListener actionNew) {
		/*App.trace(TT+".initListBox(panel, mainFrame"+
				", type="+type.toString()+
				", title="+ title+
				", list nb="+(list!=null?list.size():"null")+
				", tosel="+(tosel==null?"null":tosel.getName())+
				", addNew="+(addNew==true?"true":"false"));*/
		JPanel p = new JPanel(new MigLayout(MIG.FILL, "[][]"));
		//p.setPreferredSize(new Dimension(1024, 768));
		if (!title.isEmpty()) {
			p.add(new JSLabel(I18N.getColonMsg(title)), MIG.SPAN);
		}
		DefaultListModel model = new DefaultListModel();
		for (Object obj : list) {
			model.addElement(obj);
		}
		JList ls = new JList(model);
		ls.setName("ls_" + title);
		ls.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		ls.setMaximumSize(SwingUtil.getScreenSize());
		JScrollPane scroll = new JScrollPane(ls);
		//scroll.setMaximumSize(SwingUtil.getScreenSize());
		SwingUtil.setMaxPreferredSize(scroll);
		p.add(scroll, MIG.get(MIG.SPAN, MIG.GROW));
		if (actionNew != null) {
			JButton bt = initButton("btAdd", "add", ICONS.K.ADD, "", actionNew);
			p.add(bt, MIG.LEFT);
			bt = initButton("btDelete", "delete", ICONS.K.DELETE, "", actionNew);
			p.add(bt, MIG.RIGHT);
		}
		return p;
	}

	public static CheckBoxPanel initCbList(JPanel panel, MainFrame mainFrame,
			Book.TYPE type, List<?> ls,
			boolean mandatory, JTabbedPane tab) {
		return initCbList(panel, mainFrame, type, type.toString().toLowerCase() + "s", ls, mandatory, tab);

	}

	@SuppressWarnings("unchecked")
	public static CheckBoxPanel initCbList(JPanel panel, MainFrame mainFrame,
			Book.TYPE type, String title, List<?> ls,
			boolean mandatory, JTabbedPane tab) {
		JPanel p = new JPanel(new MigLayout(MIG.get(MIG.INS1)));
		//p.setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
		p.setBorder(BorderFactory.createTitledBorder(I18N.getColonMsg(title)));
		CheckBoxPanel cb = new CheckBoxPanel(mainFrame, type);
		cb.setName(title);
		if (ls != null) {
			cb.selectEntities((List<AbstractEntity>) ls);
		}
		JScrollPane scroller = new JScrollPane(cb);
		if (tab != null) {
			scroller.setMinimumSize(new Dimension(120, 350));
			scroller.setPreferredSize(Toolkit.getDefaultToolkit().getScreenSize());
			scroller.setMaximumSize(Toolkit.getDefaultToolkit().getScreenSize());
		} else {
			scroller.setMinimumSize(new Dimension(120, 220));
		}
		p.add(scroller, MIG.get(MIG.SPAN, MIG.GROW, MIG.WRAP));

		JButton btEdit = initButton("btEdit", "edit", ICONS.K.EDIT, "edit");
		btEdit.addActionListener((ActionEvent arg0) -> {
			List<AbstractEntity> xls = cb.getSelectedEntities();
			AbstractEntity e = cb.getPointedEntity();
			if (e == null) {
				return;
			}
			boolean rc = mainFrame.showEditorAsDialog(e, btEdit);
			if (!rc) {
				cb.refresh();
				cb.selectEntities(xls);
			}
		});
		p.add(btEdit, MIG.get(MIG.SPLIT2, MIG.RIGHT));

		JButton btAdd = initButton("btAdd", "new", ICONS.K.PLUS, "new");
		btAdd.addActionListener((ActionEvent act) -> {
			//App.trace("add new entity");
			List<AbstractEntity> xls = cb.getSelectedEntities();
			AbstractEntity e = BookUtil.getNewEntity(type);
			if (e == null) {
				return;
			}
			boolean rc = mainFrame.showEditorAsDialog(e, btAdd);
			if (!rc) {
				cb.reload();
				cb.refresh();
				cb.selectEntities(xls);
			}
		});
		p.add(btAdd);

		if (tab != null) {
			addField(panel, title, "", p, !GROW, mandatory, tab);
		} else {
			panel.add(p, MIG.GROW);
		}
		return (cb);
	}

	public static JRadioButton initRadio(ButtonGroup bg, String name, String texte, boolean isCheck) {
		JRadioButton rb = new JRadioButton();
		rb.setName(name);
		rb.setText(I18N.getMsg(texte));
		rb.setSelected(isCheck);
		bg.add(rb);
		return rb;
	}

	@SuppressWarnings("unchecked")
	public static JComboBox initStatus(JPanel panel, int sel, boolean mandatory) {
		//App.trace(TT + ".initStatus(panel, sel=" + sel + ")");
		JComboBox cb = new JComboBox();
		cb.setName("cbStatus");
		cb.setRenderer(new LCRStatus());
		ComboUtil.loadCbStatus(cb, sel);
		addField(panel, "status", "", cb, !GROW, mandatory, null);
		return (cb);
	}

	@SuppressWarnings("unchecked")
	public static JTextArea initRelations(JPanel p, MainFrame mainFrame, AbstractEntity entity, AbstractEditor editor) {
		//App.trace(TT+".initRelations(editor, mainFrame, item="+item.getName()+")");
		//JPanel r = new JPanel(new MigLayout(MIG.FILL));
		p.add(new JLabel(I18N.getColonMsg("relations")), MIG.RIGHT);
		List<String> list = new ArrayList<>();
		for (Relationship relation : EntityUtil.findRelation(mainFrame, entity)) {
			list.add(relation.getName());
		}
		JTextArea ta = Ui.MultiLineLabel(ListUtil.join(list, ", "), true);
		//lb.setColumns(64);
		ta.setMinimumSize(new Dimension(16, 16));
		p.add(ta, MIG.get(MIG.SPLIT2, MIG.GROWX));
		JButton bt = initButton("btRelation", "", ICONS.K.ENT_RELATION, "relation.modify");
		bt.addActionListener(editor);
		p.add(bt, MIG.RIGHT);
		return ta;
	}

	public static JButton initButton(String name, DATA data, ICONS.K icon, String tooltip, ActionListener... al) {
		return initButton(name, data.i18n(), icon, tooltip, al);
	}

	public static JButton initButton(String name, String text, ICONS.K icon, String tooltip, ActionListener... al) {
		//App.trace(TT+".initButton(name=" + name + ", text=" + text + ", icon=" + icon.toString() + ")");
		JButton btn = new JButton();
		btn.setName(name);
		if (text != null) {
			if (text.equals("...")) {
				btn.setText(text);
			} else if (!text.isEmpty() && !text.equals(DATA.NIL.i18n())) {
				btn.setText(I18N.getMsg(text));
			}
		}
		if (icon != ICONS.K.NONE) {
			btn.setIcon(IconUtil.getIconSmall(icon));
			if (text != null && text.isEmpty()) {
				btn.setMargin(new Insets(0, 0, 0, 0));
			}

		}
		if (!tooltip.isEmpty()) {
			btn.setToolTipText(I18N.getMsg(tooltip));
		}
		if (al != null && al.length > 0) {
			btn.addActionListener(al[0]);
		}
		return (btn);
	}

	/**
	 * add a JMenu
	 *
	 * @param str
	 * @return
	 */
	public static JMenu addMenu(String str) {
		String t[] = str.split(";");
		JMenu menu = new JMenu(t[0]);
		if (t.length > 1) {
			menu.setMnemonic(t[1].charAt(0));
		}
		return menu;
	}

	/**
	 * get a JMenuItem
	 *
	 * @param str: format is text;mnemonic;tooltip;accel
	 * @param action
	 * @return
	 */
	public static JMenuItem addMenuItem(String str, ActionListener... action) {
		String s[] = str.split(";");
		int accel = 0;
		int mask = 0;
		if (s.length > 3 && !s[3].isEmpty()) {
			if (s[3].startsWith("Ctrl")) {
				mask = InputEvent.CTRL_DOWN_MASK;
			}
			if (s[3].startsWith("Alt")) {
				mask = InputEvent.ALT_DOWN_MASK;
			}
			if (s[3].startsWith("Ctrl+Alt")) {
				mask = InputEvent.CTRL_DOWN_MASK & InputEvent.ALT_DOWN_MASK;
			}
			accel = s[3].charAt(s[3].length() - 1);
		}
		char mnem = ' ';
		if (s.length > 1) {
			mnem = s[1].charAt(0);
		}
		String tip = "";
		if (s.length > 2) {
			tip = s[2];
		}
		return initMenuItem(s[0], mnem, tip, accel, mask, action);
	}

	public static JMenuItem initMenuItem(String text, char mnemonic, String tip, int accel, int mask, ActionListener... action) {
		JMenuItem mi = new JMenuItem(text.trim());
		if (mnemonic != ' ') {
			mi.setMnemonic(mnemonic);
		}
		if (!tip.isEmpty()) {
			mi.setToolTipText(tip.trim());
		}
		if (accel > 0) {
			mi.setAccelerator(javax.swing.KeyStroke.getKeyStroke(accel, mask));
		}
		if (action != null) {
			mi.addActionListener(action[0]);
		}
		return mi;
	}

	public static JTextArea MultiLineLabel(String text) {
		return MultiLineLabel(text, false);
	}

	public static JTextArea MultiLineLabel(String text, boolean border) {
		JLabel lb = new JLabel();
		JTextArea ta = new JTextArea(text);
		ta.setFont(FontUtil.getItalicFont(lb.getFont()));
		ta.setWrapStyleWord(true);
		ta.setLineWrap(true);
		ta.setEditable(false);
		if (border) {
			JTextField tf = new JTextField();
			ta.setBorder(tf.getBorder());
		}
		ta.setBackground(new Color(lb.getBackground().getRGB(), true));
		ta.setForeground(new Color(lb.getForeground().getRGB(), true));
		ta.setOpaque(lb.isOpaque());
		return ta;
	}

}
